/***********************************************************************************************************************
 *
 * blueBill Core - open source birding
 * Copyright (C) 2009-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluebill.tidalwave.it
 * SCM: https://java.net/hg/bluebill~core-src
 *
 **********************************************************************************************************************/
package it.tidalwave.observation;

import javax.annotation.Nonnull;

/***********************************************************************************************************************
 *
 * A visitor for an {@link ObservationSet} which contains callback methods to navigate through the data.
 *
 **********************************************************************************************************************/
public interface ObservationVisitor
  {
    public static final Class<ObservationVisitor> ObservationVisitor = ObservationVisitor.class;
    
    /*******************************************************************************************************************
     *
     * Visits an {@link ObservationSet}. This method is called before visiting single {@link Observation}s (pre-order).
     * 
     * @param  observationSet  the visited {@code ObservationSet}
     *
     ******************************************************************************************************************/
    public void preVisit (@Nonnull ObservationSet observationSet);

    /*******************************************************************************************************************
     *
     * Visits an {@link ObservationSet}. This method is actually called just after {@link #preVisit()}, it makes sense 
     * to implement it when you don't need to distinguish between pre-order and post-order traversal.
     * 
     * @param  observationSet  the visited {@code ObservationSet}
     *
     ******************************************************************************************************************/
    public void visit (@Nonnull ObservationSet observationSet);
    
    /*******************************************************************************************************************
     *
     * Visits an {@link ObservationSet}. This method is called after visiting single {@link Observation}s (post-order).
     * 
     * @param  observationSet  the visited {@code ObservationSet}
     *
     ******************************************************************************************************************/
    public void postVisit (@Nonnull ObservationSet observationSet);

    /*******************************************************************************************************************
     *
     * Visits an {@link Observation}. This method is called before visiting single {@link ObservationItem}s (pre-order).
     * 
     * @param  observation  the visited {@code Observation}
     *
     ******************************************************************************************************************/
    public void preVisit (@Nonnull Observation observation);

    /*******************************************************************************************************************
     *
     * Visits an {@link Observation}. This method is actually called just after {@link #preVisit()}, it makes sense 
     * to implement it when you don't need to distinguish between pre-order and post-order traversal.
     * 
     * @param  observation  the visited {@code Observation}
     *
     ******************************************************************************************************************/
    public void visit (@Nonnull Observation observation);
    
    /*******************************************************************************************************************
     *
     * Visits an {@link Observation}. This method is called after visiting single {@link ObservationItem}s (post-order).
     * 
     * @param  observation  the visited {@code Observation}
     *
     ******************************************************************************************************************/
    public void postVisit (@Nonnull Observation observation);

    /*******************************************************************************************************************
     *
     * Visits an {@link ObservationItem}. 
     * 
     * @param  observationItem  the visited {@code ObservationItem}
     *
     ******************************************************************************************************************/
    public void visit (@Nonnull ObservationItem observationItem);
  }

